/*
 * Copyright (c) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.util;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.fail;

import java.util.UUID;

import org.junit.jupiter.api.Test;
import org.openepics.names.exception.InputNotAvailableException;
import org.openepics.names.rest.beans.Type;

/**
 * Unit tests for ValidateUtil class.
 *
 * @author Lars Johansson
 *
 * @see ValidateUtil
 */
class ValidateUtilTest {

    /**
     * Test of validate input comment.
     */
    @Test
    void validateInputCommentNull() {
        try {
            ValidateUtil.validateInputComment(null);
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals(null, e.getDetails());
            assertEquals("comment", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input comment.
     */
    @Test
    void validateInputCommentEmpty() {
        try {
            ValidateUtil.validateInputComment("");
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals("", e.getDetails());
            assertEquals("comment", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input comment.
     */
    @Test
    void validateInputComment() {
        ValidateUtil.validateInputComment("asdf");
    }

    /**
     * Test of validate input description.
     */
    @Test
    void validateInputDescriptionNull() {
        try {
            ValidateUtil.validateInputDescription(null);
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals(null, e.getDetails());
            assertEquals("description", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input description.
     */
    @Test
    void validateInputDescriptionEmpty() {
        try {
            ValidateUtil.validateInputDescription("");
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals("", e.getDetails());
            assertEquals("description", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input description.
     */
    @Test
    void validateInputDescription() {
        ValidateUtil.validateInputDescription("asdf");
    }

    /**
     * Test of validate input mnemonic.
     */
    @Test
    void validateInputMnemonicNull() {
        try {
            ValidateUtil.validateInputMnemonic(null);
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals(null, e.getDetails());
            assertEquals("mnemonic", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input mnemonic.
     */
    @Test
    void validateInputMnemonicEmpty() {
        try {
            ValidateUtil.validateInputMnemonic("");
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals("", e.getDetails());
            assertEquals("mnemonic", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input mnemonic.
     */
    @Test
    void validateInputMnemonic() {
        ValidateUtil.validateInputMnemonic("asdf");
    }

    /**
     * Test of validate input name.
     */
    @Test
    void validateInputNameNull() {
        try {
            ValidateUtil.validateInputName(null);
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals(null, e.getDetails());
            assertEquals("name", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input name.
     */
    @Test
    void validateInputNameEmpty() {
        try {
            ValidateUtil.validateInputName("");
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals("", e.getDetails());
            assertEquals("name", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input name.
     */
    @Test
    void validateInputName() {
        ValidateUtil.validateInputName("asdf");
    }

    /**
     * Test of validate input type.
     */
    @Test
    void validateInputTypeNull() {
        try {
            ValidateUtil.validateInputType(null);
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals(null, e.getDetails());
            assertEquals("type", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input type.
     */
    @Test
    void validateInputType() {
        ValidateUtil.validateInputType(Type.SYSTEMGROUP);
        ValidateUtil.validateInputType(Type.SYSTEM);
        ValidateUtil.validateInputType(Type.SUBSYSTEM);
        ValidateUtil.validateInputType(Type.DISCIPLINE);
        ValidateUtil.validateInputType(Type.DEVICEGROUP);
        ValidateUtil.validateInputType(Type.DEVICETYPE);
    }

    /**
     * Test of validate input uuid.
     */
    @Test
    void validateInputUuidNull() {
        try {
            ValidateUtil.validateInputUuid(null);
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals(null, e.getDetails());
            assertEquals("uuid", e.getField());
            assertEquals(null, e.getCause());
        } catch (Exception e) {
            fail();
        }
    }
    /**
     * Test of validate input uuid.
     */
    @Test
    void validateInputUuidEmpty() {
        try {
            ValidateUtil.validateInputUuid("");
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals("", e.getDetails());
            assertEquals("uuid", e.getField());
            assertEquals(null, e.getCause());
        }
    }
    /**
     * Test of validate input uuid.
     */
    @Test
    void validateInputUuid() {
        ValidateUtil.validateInputUuid(UUID.randomUUID().toString());
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Test of validate input read names.
     */
    @Test
    void validateInputReadNames() {
        ValidateNameElementUtil.validateNamesInputRead(
                null, null, null, null, null, null, null,
                null, null, null, null, null);
    }

    /**
     * Test of validate input create name element.
     */
    @Test
    void validateInputCreateNameElement() {
        ValidateNameElementUtil.validateNameElementInputCreate(null);
    }

    /**
     * Test of validate data create name element.
     */
    @Test
    void validateDataCreateNameElement() {
        ValidateNameElementUtil.validateNameElementDataCreate(null, null, null, null, null);
    }

    /**
     * Test of validate data create name.
     */
    @Test
    void validateDataCreateName() {
        ValidateNameElementUtil.validateNameDataCreate(null, null, null, null);
    }
    /**
     * Test of validate input update name element.
     */
    @Test
    void validateInputUpdateNameElement() {
        ValidateNameElementUtil.validateNameElementInputUpdate(null);
    }

    /**
     * Test of validate data update name element.
     */
    @Test
    void validateDataUpdateNameElement() {
        ValidateNameElementUtil.validateNameElementDataUpdate(null, null, null, null, null);
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Test of validate input read structures.
     */
    @Test
    void validateInputReadStructuresTypeNull() {
        try {
            ValidateStructureElementUtil.validateStructuresInputRead(
                    null, null, null,
                    null, null, null, null, null, null, null,
                    null, null, null, null, null);
            fail();
        } catch (InputNotAvailableException e) {
            assertEquals(TextUtil.VALUE_IS_NOT_AVAILABLE, e.getMessage());
            assertEquals(null, e.getCause());
        }
    }

    /**
     * Test of validate input read structures.
     */
    @Test
    void validateInputReadStructures() {
        ValidateStructureElementUtil.validateStructuresInputRead(
                Type.SYSTEMGROUP, null, null,
                null, null, null, null, null, null, null,
                null, null, null, null, null);
        ValidateStructureElementUtil.validateStructuresInputRead(
                Type.SYSTEM, null, null,
                null, null, null, null, null, null, null,
                null, null, null, null, null);
        ValidateStructureElementUtil.validateStructuresInputRead(
                Type.SUBSYSTEM, null, null,
                null, null, null, null, null, null, null,
                null, null, null, null, null);
        ValidateStructureElementUtil.validateStructuresInputRead(
                Type.DISCIPLINE, null, null,
                null, null, null, null, null, null, null,
                null, null, null, null, null);
        ValidateStructureElementUtil.validateStructuresInputRead(
                Type.DEVICEGROUP, null, null,
                null, null, null, null, null, null, null,
                null, null, null, null, null);
        ValidateStructureElementUtil.validateStructuresInputRead(
                Type.DEVICETYPE, null, null,
                null, null, null, null, null, null, null,
                null, null, null, null, null);
    }

    /**
     * Test of validate data create structure element.
     */
    @Test
    void validateDataCreateStructureElement() {
        ValidateStructureElementUtil.validateStructureElementDataCreate(null, null, null, null);
    }

    /**
     * Test of validate data create structure.
     */
    @Test
    void validateDataCreateStructuret() {
        ValidateStructureElementUtil.validateStructureDataCreate(null, null, null, null, null);
    }

    /**
     * Test of validate input update structure element.
     */
    @Test
    void validateInputUpdateStructureElement() {
        ValidateStructureElementUtil.validateStructureElementInputUpdate(null, null);
    }

    /**
     * Test of validate data update structure element.
     */
    @Test
    void validateDataUpdateStructureElement() {
        ValidateStructureElementUtil.validateStructureElementDataUpdate(null, null, null, null);
    }

}
